/**********/

FILE *open_file_r (char *name)
{
  FILE *fp;
  if ((fp = fopen (name, "r")) == NULL)
    {
      printf ("ERROR. Could not open file %s.\n.Exiting.\n", name);
      exit (1);
    }
  return (fp);
}

/**********/

FILE *open_file_w (char *name)
{
  FILE *fp;
  if ((fp = fopen (name, "w")) == NULL)
    {
      printf ("ERROR. Could not open file %s.\n.Exiting.\n", name);
      exit (1);
    }
  return (fp);
}

/**********/

FILE *open_file_a (char *name)
{
  FILE *fp;
  if ((fp = fopen (name, "a")) == NULL)
    {
      printf ("ERROR. Could not open file %s.\n.Exiting.\n", name);
      exit (1);
    }
  return (fp);
}

/**********/

void close_file (FILE * fp)
{
  fclose (fp);
}

/**********/

void read_pdb_file(struct protein *reference_protein)
{
  int i, discard_alternative_position,advance_counter_flag;
  char line[200], *a;
  char field[10];
  FILE *fp;
  char aa_index[10], current_aa_index[10];

  /* PDB FORMAT 

COLUMNS        DATA TYPE     FIELD        DEFINITION 
------------------------------------------------------------------------------------- 
 1 -  6        Record name   "ATOM  " 
 
 7 - 11        Integer       serial       Atom serial number. 
 
13 - 16        Atom          name         Atom name. 
 
17             Character     altLoc       Alternate location indicator. 
 
18 - 20        Residue name  resName      Residue name. 
 
22             Character     chainID      Chain identifier. 
 
23 - 26        Integer       resSeq       Residue sequence number. 
 
27             AChar         iCode        Code for insertion of residues. 
 
31 - 38        Real(8.3)     x            Orthogonal coordinates for X in Angstroms. 
 
39 - 46        Real(8.3)     y            Orthogonal coordinates for Y in Angstroms. 
 
47 - 54        Real(8.3)     z            Orthogonal coordinates for Z in Angstroms. 
 
55 - 60        Real(6.2)    occupancy     Occupancy. 
 
61 - 66        Real(6.2)    tempFactor    Temperature factor. 
  
77 - 78        LString(2)   element       Element symbol, right-justified. 
 
79 - 80        LString(2)   charge        Charge on the atom. 

  */

  /* Find out length of protein */

  printf("Will attempt to read file %s\n",reference_protein->name);
  fp = open_file_r(reference_protein->name);
  (reference_protein->length)=0;
  while (fgets(line, 200, fp)){
    if (strncmp (line, "ENDMDL", 6)==0){
      printf("Stopping at end of first model.\n");
      break;
    }
    /* check if line buffer begins with "ATOM" */
    if (!strncmp (line, "ATOM", 4)){
      a = line +12; /* advance line buffer and check if 
                       we have a CA atom */
      /* check if character at position 16 is different from ' '
         or 'A'. If so, we are facing an alternative position
         and we will not consider it */
      discard_alternative_position=1;
      if (line[16]==' ') discard_alternative_position=0;
      if (line[16]=='A') discard_alternative_position=0;

      if ((!strncmp (a, " CA ", 4)) && (discard_alternative_position==0)){
        (reference_protein->length)++;
      }
    }
  }
  rewind(fp);

  if (reference_protein->length==0){
    printf("Error! Found no CA atoms.\nPlease check PDB formatting to see if ATOM and CA strings start at columns 1 and 13, respectively.\n");
    exit(1);
  }

  /* and allocate it */
  proteinalloc(reference_protein->length,reference_protein);


  /* Let's allocate the memory for the protein */
  i=-1; /* default value when we have not read any amino acid yet */
  strcpy(aa_index,"XXXXX"); /* fake index to mean that we have not read any amino acid */

  while (fgets(line, 200, fp)){
    if (strncmp (line, "ENDMDL", 6)==0){
      printf("Stopping at end of first model.\n");
      break;
    }
    /* check if line buffer begins with "ATOM" */
    if (!strncmp (line, "ATOM", 4)){
      
      /* check if character at position 16 is different from ' '
         or 'A'. If so, we are facing an alternative position
         and we will not consider it */
      discard_alternative_position=1;
      if (line[16]==' ') discard_alternative_position=0;
      if (line[16]=='A') discard_alternative_position=0;
      if (discard_alternative_position==1) continue;

      a=line+22;
      /* the index can be followed by chain indicators as in pdb 1mfa.
         we then read only the numerical characters */
      
      strncpy(field,a,5);field[5]='\0';
      strcpy(current_aa_index,field);
     
      if (strncmp(aa_index,current_aa_index,5)!=0) { /* check if a.a. index has changed */
        strcpy(aa_index,current_aa_index); 
        advance_counter_flag=0;
        if (i==-1)  advance_counter_flag=1;
        if ((i>=0) &&  ((strncmp (reference_protein->residue[i].original_CA_index, "XXXXX", 5)!=0))) advance_counter_flag=1;
        if (advance_counter_flag==1){
          /* we have correctly read the previous CA; let's advance the index */
          /* otherwise don't advance the index as this means that the PDB files contained some
             atoms of the amino acids but not the CA. In this case we skip all these "orphan atoms"
             by not advancing the index and by resetting the residue variables to the default values */
          i++;
          if (i >= reference_protein->length){
            /* we must capture the CA-orphan a.a. at the end of the file here as otherwise
               we would begin to assign their atoms to memory positions that are unavailable ! */
            printf("\nWARNING: Amino acid %4s of file %s has no CA. The atomic entries of this amino acid (and subsequence ones, if any) will be skipped\n",
                   aa_index,reference_protein->original_protname);
            goto end;
        }

        }
        else{
          printf("\nWARNING: amino acid %4s of file %s has no CA. The atomic entries of this amino acid will be skipped\n",
            aa_index,reference_protein->original_protname);
          printf("i: %4d  original_CA_index %4s  buffer : %s\n",i,reference_protein->residue[i].original_CA_index,a);
        }
        reference_protein->residue[i].n_atoms=0;
        strcpy(reference_protein->residue[i].original_CA_index,"XXXXX"); /* signal that we have not read any CA atom, yet */
      }
      else { /*check if, despite the a.a. index has not changed and despite we have already read a CA atom, there is a second CA atom. See e.g. 1bgw, a.a. 183 */
        if (((strncmp (reference_protein->residue[i].original_CA_index, "XXXXX", 5)!=0)) && (line[13]=='C') && (line[14]=='A')){
          printf("\nWARNING: amino acid %4s of file %s appears to have multiple CA's.\n",  aa_index,reference_protein->original_protname);
          advance_counter_flag=1;
          i++; /* advance "our" CA index */
          if (i >= reference_protein->length){
            /* we must capture the CA-orphan a.a. at the end of the file here as otherwise
               we would begin to assign their atoms to memory positions that are unavailable ! */
            printf("\nWARNING: Amino acid %4s of file %s has no CA. The atomic entries of this amino acid (and subsequence ones, if any) will be skipped\n",
                   aa_index,reference_protein->original_protname);
            goto end;
          }

          reference_protein->residue[i].n_atoms=0;
          strcpy(reference_protein->residue[i].original_CA_index,"XXXXX"); /* signal that we have not read any CA atom, yet */
        }
      }
      
      /* treat CA in a special way */
      a = line +12; /* advance line buffer and check if we have a CA atom */
      if ((!strncmp (a, " CA ", 4))){
        a = line +17;
        strncpy(field,a,4);field[4]='\0';
        sscanf(field,"%s",&(reference_protein->residue[i].res_type[0]));
        /* the chain index can be just next to the residue numbering
           as in file 1bmv. We need to read it from the column. */
        reference_protein->residue[i].original_chain=line[21];
        a=line+22;
        /* the index can be followed by chain indicators as in pdb 1mfa.
           we then read to red only the numerical characters */

        strncpy(field,a,5);field[5]='\0';
        strcpy(reference_protein->residue[i].original_CA_index,field);
        a=line+30;
        strncpy(field,a,8);field[8]='\0';
        if (sscanf(field,"%f",&(reference_protein->residue[i].ca[0]))!=1){printf("Fatal error. Could not read CA coordinate in protein %s.\nLine %s\n",reference_protein->original_protname,line); exit(0);}
        a=line+38;
        strncpy(field,a,8);field[8]='\0';
        if (sscanf(field,"%f",&(reference_protein->residue[i].ca[1]))!=1){printf("Fatal error. Could not read CA coordinate in protein %s.Line %s\n\n",reference_protein->original_protname,line); exit(0);}
        a=line+46;
        strncpy(field,a,8);field[8]='\0';
        if (sscanf(field,"%f",&(reference_protein->residue[i].ca[2]))!=1){printf("Fatal error. Could not read CA coordinate in protein %s.\nLine %s\n",reference_protein->original_protname,line); exit(0);}

        /* Finally read B factor */
        a= line + 60;
        strncpy(field,a,6);field[6]='\0';
            if ( sscanf(field,"%f",&(reference_protein->residue[i].Bfactor)) !=1) {reference_protein->residue[i].Bfactor=0.0; }
      }

       /* update atomic index, type and coordinates (including CA's again */
      if (reference_protein->residue[i].n_atoms >= MaxNatoms){
        printf("\nWARNING. Amino acid %4s of file %s seems to have too many atoms. I will keep only the first %3d\n",aa_index,reference_protein->original_protname,MaxNatoms);
        goto skip_reading_atomic_details;
      }

      a = line +6;
      strncpy(field,a,5);field[5]='\0';
      if (sscanf(field,"%d",&(reference_protein->residue[i].atom_original_index[reference_protein->residue[i].n_atoms]))!=1){printf("Fatal error. Could not read atomic index in protein %s.\nLine %s\n",reference_protein->original_protname,line); exit(0);}
      a = line +12;
      strncpy(field,a,4);field[4]='\0';
      strncpy(&(reference_protein->residue[i].atom_type[reference_protein->residue[i].n_atoms][0]),field,5);
      a=line+30;
      strncpy(field,a,8);field[8]='\0';
      if (sscanf(field,"%f",&(reference_protein->residue[i].atom_coordinates[reference_protein->residue[i].n_atoms][0]))!=1){printf("Fatal error. Could not read atomic coordinate in protein %s.\nLine %s\n",reference_protein->original_protname,line); exit(0);}
      a=line+38;
      strncpy(field,a,8);field[8]='\0';
      if (sscanf(field,"%f",&(reference_protein->residue[i].atom_coordinates[reference_protein->residue[i].n_atoms][1]))!=1){printf("Fatal error. Could not read CA coordinate in protein %s.\nLine %s\n",reference_protein->original_protname,line); exit(0);}
      a=line+46;
      strncpy(field,a,8);field[8]='\0';
      if (sscanf(field,"%f",&(reference_protein->residue[i].atom_coordinates[reference_protein->residue[i].n_atoms][2]))!=1){printf("Fatal error. Could not read CA coordinate in protein %s.\nLine %s\n",reference_protein->original_protname,line); exit(0);}
      reference_protein->residue[i].n_atoms++;

    skip_reading_atomic_details:
      ;
    }
  }
  fclose(fp);
  advance_counter_flag=0;
  /* before advancing the number of a.a. let's us check that the last a.a. is not orphan of its CA (see e.g. 1u0OC, 2iz1A, ...) */
  if (i==-1)  advance_counter_flag=1;
  if ((i>=0) &&  ((strncmp (reference_protein->residue[i].original_CA_index, "XXXXX", 5)!=0))) advance_counter_flag=1;
  if (advance_counter_flag==1) i++; 

 end:
 if (i != (reference_protein->length)){
    printf("ERROR. Inconsistent number of CA atoms between first and second pass of pdb file %s (%d vs %d).\nPlease report this problem to C. Micheletti: michelet@sissa.it\n",
           reference_protein->original_protname,i,(reference_protein->length));
    exit(0);
  }
}


/******************************************/
void read_input_arguments_and_load_proteins(int argc, char **argv, struct protein *reference_protein ){
  
  int itemp;
  int j, n_proteins;
  long ltime;
  int stime;
  
  Max_protein_length=0;
  
  if ( (argc != 2)  ){
    fprintf(stderr,"ERROR: wrong number of arguments.\nUsage: %s pdbfile \n\n",argv[0]);
  }
  
  
  
  /* Defaults values set at beginning of main program */
  
  n_proteins=0;
  
  printf("argv %s %s\n",argv[0],argv[1]);
  
  sprintf(reference_protein[n_proteins].name,"%s",argv[1]);
  sprintf(reference_protein[n_proteins].original_protname,"%s",argv[1]);
  n_proteins++;
  
    
    
  printf("Start loading data for input protein %s\n",reference_protein[0].original_protname);
  read_pdb_file(&(reference_protein[0]));
  printf("\t  PDBfile %s contains %3d residues. PDBfile was read correctly.\n",reference_protein[0].original_protname,reference_protein[0].length);
  if (reference_protein[0].length > Max_protein_length) Max_protein_length=reference_protein[0].length;
  
  printf("Finished loading data for the input protein\n\n");   

}
/**************/

void write_pdb_structure (char *outputfilename, struct protein protein, int domain_index[],int flag_CA_only){

  int i, j;
  FILE *fp;

  fp=fopen(outputfilename,"w");
  fprintf(fp,"REMARK %15s\n",protein.original_protname);
          
  for(i=0; i < protein.length;i++){
     if ( flag_CA_only==1){
       fprintf(fp,"ATOM  %5d  CA  %3s %c%5s   %8.3f%8.3f%8.3f  1.00 %5.2f\n",
               i+1,protein.residue[i].res_type,protein.residue[i].original_chain,protein.residue[i].original_CA_index,protein.residue[i].ca[0],protein.residue[i].ca[1],protein.residue[i].ca[2],1.0*domain_index[i]);
     }
     else  {
       for(j=0; j < protein.residue[i].n_atoms; j++){
       fprintf(fp,"ATOM  %5d %-4s %3s %c%5s   %8.3f%8.3f%8.3f  1.00 %5.2f\n",
               protein.residue[i].atom_original_index[j],protein.residue[i].atom_type[j],protein.residue[i].res_type,protein.residue[i].original_chain,protein.residue[i].original_CA_index,protein.residue[i].atom_coordinates[j][0],protein.residue[i].atom_coordinates[j][1],protein.residue[i].atom_coordinates[j][2],1.0*domain_index[i]);
       }
       
     }

  }
  fflush(fp);                                   
  fclose(fp);
}


/**************/

void write_domain_index (char *outputfilename, struct protein protein, int domain_index[]) {

  int i, j;
  FILE *fp;

  fp = fopen(outputfilename,"w");
  fprintf(fp,"# residue \tlabel \n");
          
  for(i=0; i < protein.length; i++){
    fprintf(fp, "%d \t%d \n", i, domain_index[i]);
  }

  fflush(fp);                                   
  fclose(fp);
}
