/* 4/3/2015 CM
programma che fa il merge di un file PDB con un file con gli indici di domini rigidi.
fatto saccheggiando pezzi dal codice di MISTRAL.

Nota: Il percorso alla directory con i subdivision files (e la radice del nome dei subdivision files) sono hardcoded nella routine io.c

CM
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "messages.lib"
#include "my_memory.lib"
#define MaxNumberProteins (1)
#define MaxProteinNameLength (200)
#define MaxNatoms (30) /* Max Number of atoms in an amino acid (for memory allocation purposes) */
char output_PDB_filename[200]="indexed_PDBfile.pdb"; /* default name for PDB file containing the alignment */

struct residue{
  float ca[3];     /* CA coordinates */
  float Bfactor;   /* B factor */
  char res_type[4];
  char original_chain;
  char original_CA_index[10];
  int n_atoms;
  char atom_type[MaxNatoms][5];
  float atom_coordinates[MaxNatoms][3];
  int  atom_original_index[MaxNatoms];
};

struct protein {
  char name[MaxProteinNameLength];
  char original_protname[MaxProteinNameLength];
  int length;
  float gyr_radius;
  struct residue *residue;
};


void 
proteinalloc (int length, struct protein *prot)
{
  prot->length = length;
  if ((prot->residue = calloc (length, sizeof (struct residue)))==NULL){
    printf("FATAL ERROR in memory allocation for protein %s of length %4d. Exiting.\n",prot->name,prot->length);
    exit(1);
  }

}

void  proteinfree(struct protein *prot)
{

  free(prot->residue);  
}


int Max_protein_length; /* Global variable storing the maximum protein length of the input set. Useful for memory allocation purposes */

#include "io.c"



int main(int argc, char **argv){

  int i, j, k, n_proteins, n_entries_in_domain_file;
  int domain_index[10000], flag_CA_only=0;
  struct protein  reference_protein[MaxNumberProteins];
  char inputfilename[200],outputfilename[200];
  FILE *fp;

  n_proteins=1;
  printf("\n\t\t*****  merges a PDB file and a subdivision file  *****\n\n");
  read_input_arguments_and_load_proteins(argc, argv, reference_protein);

  for(k=2; k<1000 ; k++){ /* scan subdivisions */
    sprintf(inputfilename,"results/final_labels_kmed-%d.dat",k);
    if ((fp=fopen(inputfilename,"r"))==NULL) continue; /* OK, not too smart.... */
    n_entries_in_domain_file=0;
    while ( fscanf(fp,"%d %d",&i,&j)!=EOF){
      domain_index[n_entries_in_domain_file]=j;
      n_entries_in_domain_file++;
    }
    fclose(fp);
//     printf("read %3d entries from file %s\n",n_entries_in_domain_file,inputfilename);

    if (n_entries_in_domain_file!=reference_protein[0].length){
      printf("FATAL ERROR. Inconsistent length of PDF and domain index file: %d vs %d\n",n_entries_in_domain_file,reference_protein[0].length);
      exit(1);
    }
   sprintf(outputfilename,"subdivision_in_%d_domains.pdb",k);
   write_pdb_structure(outputfilename, reference_protein[0], domain_index, flag_CA_only);
   sprintf(outputfilename,"subdivision_in_%d_domains-labels.dat",k);
   write_domain_index(outputfilename, reference_protein[0], domain_index);
//    printf("written output to file %s\n\n",outputfilename);
    
  }
//   printf("Done.\n");

  for(i=0; i < n_proteins; i++){
    proteinfree(&(reference_protein[i]));
  }
 

  exit(0);
}
